# -*- coding: cp932 -*-

# t[}쐬c[ for ArcGIS 10.4
#  (c) Takashi Kirimura  2016/12/17

import arcpy
from arcpy import env
import math
import os.path

class Toolbox(object):
    def __init__(self):
        self.label = u"DrawFlowPolyline"
        self.alias = u"DrawFlowPolyline"

        # List of tool classes associated with this toolbox
        self.tools = [CreateFlowMap] 

class CreateFlowMap(object):
    def __init__(self):
        self.label       = u"t[}i|Cj̍쐬"
        self.description = u"n_EI_̍Wle[uf[^At[|쐬܂B" + \
                           u"|ĆAn_EI_Ԃ̋𔼌aƂ鎞v̉~̉~ʂƂč쐬܂B"

    def getParameterInfo(self):
        #Define parameter definitions

        # ̓e[u
        in_table = arcpy.Parameter(
            displayName=u"̓e[u܂̓tB[`",
            name="in_table",
            datatype="GPTableView",
            parameterType="Required",
            direction="Input")
        
        # n_x tB[h
        startxvalue_field = arcpy.Parameter(
            displayName=u"n_xW",
            name="startxvalue_field",
            datatype="Field",
            parameterType="Required",
            direction="Input")
        startxvalue_field.parameterDependencies = [in_table.name]

        # n_y tB[h
        startyvalue_field = arcpy.Parameter(
            displayName=u"n_yW",
            name="startyvalue_field",
            datatype="Field",
            parameterType="Required",
            direction="Input")
        startyvalue_field.parameterDependencies = [in_table.name]

        # I_x tB[h
        endxvalue_field = arcpy.Parameter(
            displayName=u"I_xW",
            name="endxvalue_field",
            datatype="Field",
            parameterType="Required",
            direction="Input")
        endxvalue_field.parameterDependencies = [in_table.name]

        # I_y tB[h
        endyvalue_field = arcpy.Parameter(
            displayName=u"I_yW",
            name="endyvalue_field",
            datatype="Field",
            parameterType="Required",
            direction="Input")
        endyvalue_field.parameterDependencies = [in_table.name]
        
        # Wl̋ԎQ
        spatialref_field = arcpy.Parameter(
            displayName=u"Wl̋ԎQ",
            name="spatialref_field",
            datatype="GPSpatialReference",
            parameterType="Required",
            direction="Input")

        # ID tB[h
        idvalue_field = arcpy.Parameter(
            displayName=u"ID",
            name="idvalue_field",
            datatype="Field",
            parameterType="Required",
            direction="Input")
        idvalue_field.parameterDependencies = [in_table.name]
        
        # o̓tB[`
        out_features = arcpy.Parameter(
            displayName="o̓tB[`",
            name="out_features",
            datatype="DEFeatureClass",
            parameterType="Required",
            direction="Output")

        parameters = [in_table, startxvalue_field, startyvalue_field, endxvalue_field, endyvalue_field, spatialref_field, idvalue_field, out_features]
        
        return parameters

    
    def execute(self, parameters, messages):
        out_name = os.path.basename(parameters[7].valueAsText)
        env.workspace = os.path.dirname(parameters[7].valueAsText)
        geometry_type = "POLYLINE"
        has_m = "DISABLED"
        has_z = "DISABLED"
        spatial_reference = parameters[5].value

        arcpy.CreateFeatureclass_management(env.workspace, out_name, geometry_type,"", has_m, has_z, spatial_reference)

        # ̓e[u̓ǂݍ
        infields = arcpy.ListFields(parameters[0].value)

        field_id = parameters[6].valueAsText
        for field in infields:
            if field.name == field_id:
                arcpy.AddField_management(out_name, field_id, field.type, field.precision, field.scale, field.length)
        field_x1 = parameters[1].valueAsText
        field_y1 = parameters[2].valueAsText
        field_x2 = parameters[3].valueAsText
        field_y2 = parameters[4].valueAsText
        for field in infields:
            if field.name == field_x1:
                arcpy.AddField_management(out_name, field_x1, field.type, field.precision, field.scale, field.length)
            elif field.name == field_y1:
                arcpy.AddField_management(out_name, field_y1, field.type, field.precision, field.scale, field.length)
            elif field.name == field_x2:
                arcpy.AddField_management(out_name, field_x2, field.type, field.precision, field.scale, field.length)
            elif field.name == field_y2:
                arcpy.AddField_management(out_name, field_y2, field.type, field.precision, field.scale, field.length)

        arcpy.MakeTableView_management(parameters[0].value, "intable")
        outfc = env.workspace + "/" + out_name
        icursor = arcpy.da.InsertCursor(outfc,["SHAPE@", field_id, field_x1, field_y1, field_x2, field_y2])

        with arcpy.da.SearchCursor("intable", [field_x1, field_y1, field_x2, field_y2, field_id]) as cursor:
            for row in cursor:
                x1 = row[0]
                y1 = row[1]
                x2 = row[2]
                y2 = row[3]
                x3 = (x1 + x2) * 0.5
                y3 = (y1 + y2) * 0.5
                l = (x2 - x3) * (x2 - x3) + (y2 - y3) * (y2 - y3)
                r = math.sqrt(l) * 2.0
                r2jo = r * r

                if (r2jo >= l):
                    d = math.sqrt(r2jo / l - 1.0)
                    dx = d * (y2 - y3)
                    dy = d * (x2 - x3)

                    c1x = x3 + dx
                    c1y = y3 - dy

                    c2x = x3 - dx
                    c2y = y3 + dy

                r11 = math.atan2((y1 - c1y), (x1 - c1x))
                r12 = math.atan2((y2 - c1y), (x2 - c1x))
                r21 = math.atan2((y1 - c2y), (x1 - c2x))
                r22 = math.atan2((y2 - c2y), (x2 - c2x))

                if (r22 - r21) > (r12 - r11) or abs(r22 - r21) > abs(r12 - r11):
                    ra = r11
                    rb = r12
                else:
                    ra = r22 + math.pi
                    rb = r21 + math.pi
                plist = arcpy.Array(arcpy.Point(x1,y1))
                for i in range(20):
                    rr = ra + (rb - ra) * ((i + 1.0) / 20.0)
                    rx = r * math.cos(rr) + c1x
                    ry = r * math.sin(rr) + c1y
                    plist.add(arcpy.Point(rx,ry))

                plist.add(arcpy.Point(x2,y2))
                polyline = arcpy.Polyline(plist)
                icursor.insertRow([polyline, row[4], row[0], row[1], row[2], row[3]])


